/**
 * CRC - Canvas Render Components (alpha)
 *
 * MIT License
 *
 * Copyright 2020 Ben Lesh <ben@benlesh.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */
export type CanvasMouseEventHandler = (e: MouseEvent) => void;
export type CanvasCompRenderer = (ctx: CanvasRenderingContext2D, eventRegistry: EventRegistry, parentId: string) => void;
export interface CanvasMouseEventConfig {
    id: string;
    include?: 'stroke' | 'fill' | 'all';
    lineWidth?: number;
}
export interface EventRegistry {
    onClick: (path: Path2D, handler: CanvasMouseEventHandler, config: CanvasMouseEventConfig) => void;
    onDblClick: (path: Path2D, handler: CanvasMouseEventHandler, config: CanvasMouseEventConfig) => void;
    onContextMenu: (path: Path2D, handler: CanvasMouseEventHandler, config: CanvasMouseEventConfig) => void;
    onMouseOver: (path: Path2D, handler: CanvasMouseEventHandler, config: CanvasMouseEventConfig) => void;
    onMouseOut: (path: Path2D, handler: CanvasMouseEventHandler, config: CanvasMouseEventConfig) => void;
    onMouseMove: (path: Path2D, handler: CanvasMouseEventHandler, config: CanvasMouseEventConfig) => void;
}
export declare function createRenderingContext(canvas: HTMLCanvasElement): {
    readonly renderContext: CanvasRenderingContext2D;
    readonly eventRegistry: EventRegistry;
    readonly subscription: {
        unsubscribe(): void;
    };
    readonly beginRender: () => void;
    readonly endRender: () => void;
};
export declare function createRenderer(canvas: HTMLCanvasElement): {
    render: (comp: CanvasCompRenderer) => void;
    remove(): void;
};
export declare function getMousePosition(e: MouseEvent): [number, number];
type FillStyle = CanvasRenderingContext2D['fillStyle'];
type StrokeStyle = CanvasRenderingContext2D['strokeStyle'];
interface LineStyle {
    strokeStyle?: StrokeStyle;
    lineWidth?: number;
}
interface PathStyle {
    fillStyle?: FillStyle;
    strokeStyle?: StrokeStyle;
    lineWidth?: number;
    cursor?: string;
}
interface PathEvents {
    onClick?: (e: MouseEvent) => void;
    onDblClick?: (e: MouseEvent) => void;
    onContextMenu?: (e: MouseEvent) => void;
    onMouseOver?: (e: MouseEvent) => void;
    onMouseOut?: (e: MouseEvent) => void;
    onMouseMove?: (e: MouseEvent) => void;
}
export declare function rect(props: {
    x: number;
    y: number;
    width: number;
    height: number;
    style?: PathStyle;
    events?: PathEvents;
}): CanvasCompRenderer;
export declare function line(props: {
    x1: number;
    y1: number;
    x2: number;
    y2: number;
    style?: LineStyle & {
        cursor?: string;
    };
    hitPadding?: number;
    events?: PathEvents;
}): CanvasCompRenderer;
export declare function verticalLine(props: {
    x: number;
    y1?: number;
    y2?: number;
    style?: LineStyle;
    hitPadding?: number;
    events?: PathEvents;
}): CanvasCompRenderer;
export declare function horizontalLine(props: {
    y: number;
    x1?: number;
    x2?: number;
    style?: LineStyle;
    hitPadding?: number;
    events?: PathEvents;
}): CanvasCompRenderer;
export declare function img(props: {
    img: HTMLImageElement;
    x: number;
    y: number;
    width?: number;
    height?: number;
    style?: LineStyle & {
        cursor?: string;
    };
    events?: PathEvents;
}): CanvasCompRenderer;
export declare function group(children: CanvasCompRenderer[]): CanvasCompRenderer;
export declare function scale(scale: number | [number, number], renderer: CanvasCompRenderer): CanvasCompRenderer;
export declare function translate(x: number, y: number, renderer: CanvasCompRenderer): CanvasCompRenderer;
export declare function rotate(angle: number, renderer: CanvasCompRenderer): CanvasCompRenderer;
export declare function transform(matrix: DOMMatrix, renderer: CanvasCompRenderer): CanvasCompRenderer;
export declare function clipRect(rect: {
    x: number;
    y: number;
    width: number;
    height: number;
}, renderer: CanvasCompRenderer): CanvasCompRenderer;
export declare function clip(path: Path2D, renderer: CanvasCompRenderer): CanvasCompRenderer;
export declare function text(props: {
    text: string;
    x: number;
    y: number;
    style: PathStyle & {
        textAlign?: CanvasTextAlign;
        textBaseline?: CanvasTextBaseline;
        overflow?: 'visible' | 'ellipsis';
        maxWidth?: number;
        maxHeight?: number;
        wordWrap?: boolean;
        font?: string;
        lineHeight?: number;
    };
    events?: PathEvents;
}): CanvasCompRenderer;
export {};
